@@ -1,6 +1,6 @@
-// Copyright (c) 2001 Peter Karlsson
+// Copyright (c) 2001-2005 Peter Karlsson
 //
-// $Id: convert.cpp,v 1.15 2001/10/27 11:42:55 peter Exp $
+// $Id: convert.cpp,v 1.19 2005/07/14 13:28:24 peterk Exp $
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 
 #include <config.h>
 
-#define _GNU_SOURCE
 #include <string.h>
 
 #include "convert.h"
@@ -47,6 +46,7 @@ const struct tablemap fidocharsets[] =
     { "LATIN-7", iso_8859_13, iso_8859_13_rev, iso_8859_13_rev_len },
     { "LATIN-8", iso_8859_14, iso_8859_14_rev, iso_8859_14_rev_len },
     { "LATIN-9", iso_8859_15, iso_8859_15_rev, iso_8859_15_rev_len },
+    { "LATIN-10", iso_8859_16, iso_8859_16_rev, iso_8859_16_rev_len },
     { "CP437", cp_437, cp_437_rev, cp_437_rev_len },
     { "CP737", cp_737, cp_737_rev, cp_737_rev_len },
     { "CP775", cp_775, cp_775_rev, cp_775_rev_len },
@@ -96,11 +96,17 @@ const struct tablemap usenetcharsets[] =
     { "iso-8859-2", iso_8859_2, iso_8859_2_rev, iso_8859_2_rev_len },
     { "iso-8859-3", iso_8859_3, iso_8859_3_rev, iso_8859_3_rev_len },
     { "iso-8859-4", iso_8859_4, iso_8859_4_rev, iso_8859_4_rev_len },
+    { "iso-8859-5", iso_8859_5, iso_8859_5_rev, iso_8859_5_rev_len },
+    { "iso-8859-6", iso_8859_6, iso_8859_6_rev, iso_8859_6_rev_len },
+    { "iso-8859-7", iso_8859_7, iso_8859_7_rev, iso_8859_7_rev_len },
+    { "iso-8859-8", iso_8859_8, iso_8859_8_rev, iso_8859_8_rev_len },
     { "iso-8859-9", iso_8859_9, iso_8859_9_rev, iso_8859_9_rev_len },
     { "iso-8859-10", iso_8859_10, iso_8859_10_rev, iso_8859_10_rev_len },
+    { "iso-8859-11", iso_8859_11, iso_8859_11_rev, iso_8859_11_rev_len },
     { "iso-8859-13", iso_8859_13, iso_8859_13_rev, iso_8859_13_rev_len },
     { "iso-8859-14", iso_8859_14, iso_8859_14_rev, iso_8859_14_rev_len },
     { "iso-8859-15", iso_8859_15, iso_8859_15_rev, iso_8859_15_rev_len },
+    { "iso-8859-16", iso_8859_16, iso_8859_16_rev, iso_8859_16_rev_len },
     { "ibm437", cp_437, cp_437_rev, cp_437_rev_len },
     { "ibm737", cp_737, cp_737_rev, cp_737_rev_len },
     { "ibm775", cp_775, cp_775_rev, cp_775_rev_len },
@@ -136,13 +142,11 @@ const struct tablemap usenetcharsets[] =
 };
 
 /**
- * Class used to decode UTF-8 data. It converts it into wchar_t Unicode for
+ * Class used to decode SBCS data. It converts it into wchar_t Unicode for
  * internal use.
  */
-class UTF8Decoder : public Decoder
+class SBCSDecoder : public Decoder
 {
-    friend class Decoder;
-
 public:
     /**
      * Decode character data.
@@ -151,9 +155,27 @@ public:
      */
     virtual wstring Decode(const string &input);
 
+    /** Constructor used only from Decoder class methods. */
+    SBCSDecoder(const unsigned short *map) : inmap(map) {};
+
 protected:
-    /** Protected constructor for internal use only. */
-    UTF8Decoder() : Decoder(NULL) {};
+    /** Conversion table. */
+    const unsigned short *inmap;
+};
+
+/**
+ * Class used to decode UTF-8 data. It converts it into wchar_t Unicode for
+ * internal use.
+ */
+class UTF8Decoder : public Decoder
+{
+public:
+    /**
+     * Decode character data.
+     * @param input Legacy encoded data to decode.
+     * @return Unicode version of data.
+     */
+    virtual wstring Decode(const string &input);
 };
 
 // Get Decoder object for a character set
@@ -170,7 +192,7 @@ Decoder *Decoder::GetDecoderByName(const char *charset)
         if (0 == fcompare(charset, fidocharsets[i].charset))
 #endif
         {
-            return new Decoder(fidocharsets[i].inmap);
+            return new SBCSDecoder(fidocharsets[i].inmap);
         }
     }
 
@@ -185,7 +207,7 @@ Decoder *Decoder::GetDecoderByName(const char *charset)
         if (0 == fcompare(charset, usenetcharsets[i].charset))
 #endif
         {
-            return new Decoder(usenetcharsets[i].inmap);
+            return new SBCSDecoder(usenetcharsets[i].inmap);
         }
     }
 
@@ -202,7 +224,7 @@ Decoder *Decoder::GetDecoderByName(const char *charset)
     }
 
     // Return first in MIME table as default
-    return new Decoder(usenetcharsets[0].inmap);
+    return new SBCSDecoder(usenetcharsets[0].inmap);
 }
 
 // Identify Fidonet character set from kludges
@@ -213,7 +235,7 @@ Decoder *Decoder::GetDecoderByKludges(const char *kludges)
     if (!chrs)
     {
         // Return first in Fido table
-        return new Decoder(fidocharsets[0].inmap);
+        return new SBCSDecoder(fidocharsets[0].inmap);
     }
 
     // Isolate kludge
@@ -256,7 +278,7 @@ Decoder *Decoder::GetDecoderByKludges(const char *kludges)
     {
         if (0 == fcompare(charset, fidocharsets[i].charset))
         {
-            return new Decoder(fidocharsets[i].inmap);
+            return new SBCSDecoder(fidocharsets[i].inmap);
         }
     }
 
@@ -267,7 +289,7 @@ Decoder *Decoder::GetDecoderByKludges(const char *kludges)
     }
 
     // Return first in Fido table
-    return new Decoder(fidocharsets[0].inmap);
+    return new SBCSDecoder(fidocharsets[0].inmap);
 }
 
 // Identify Usenet character set from headers
@@ -301,7 +323,7 @@ Decoder *Decoder::GetDecoderByMIMEHeaders(const char *headers)
     if (!charset)
     {
         // Return first in MIME table
-        return new Decoder(usenetcharsets[0].inmap);
+        return new SBCSDecoder(usenetcharsets[0].inmap);
     }
 
     // Isolate charset parameter
@@ -325,15 +347,18 @@ Decoder *Decoder::GetDecoderByMIMEHeaders(const char *headers)
     const char *p;
     if (NULL != (p = strchr(start, ' ')))
     {
-        next = next <? p;
+    	if (p < next)
+    		next = p;
     }
     if (NULL != (p = strchr(start, 1)))
     {
-        next = next <? p;
+    	if (p < next)
+    		next = p;
     }
     if (NULL != (p = strchr(start, ',')))
     {
-        next = next <? p;
+    	if (p < next)
+    		next = p;
     }
 
     string charsetstring(start, next - start);
@@ -343,7 +368,7 @@ Decoder *Decoder::GetDecoderByMIMEHeaders(const char *headers)
     {
         if (0 == fcompare(charsetstring, usenetcharsets[i].charset))
         {
-            return new Decoder(usenetcharsets[i].inmap);
+            return new SBCSDecoder(usenetcharsets[i].inmap);
         }
     }
 
@@ -354,11 +379,11 @@ Decoder *Decoder::GetDecoderByMIMEHeaders(const char *headers)
     }
 
     // Return first in MIME table
-    return new Decoder(usenetcharsets[0].inmap);
+    return new SBCSDecoder(usenetcharsets[0].inmap);
 }
 
 // Decode legacy character string
-wstring Decoder::Decode(const string &input)
+wstring SBCSDecoder::Decode(const string &input)
 {
 #ifdef HAVE_WORKING_WSTRING
     wstring s;
@@ -458,6 +483,32 @@ wstring UTF8Decoder::Decode(const string &input)
     return s;
 }
 
+/**
+ * Class used to encode character data into a legacy character set. It converts
+ * it from Unicode for external use.
+ */
+class SBCSEncoder : public Encoder
+{
+public:
+    /**
+     * Encode character data.
+     * @param input Unicode data to encode.
+     * @return Legacy version of data.
+     */
+    virtual string Encode(const wstring &input);
+
+    /** Constructor used only from Encoder class methods. */
+    SBCSEncoder(const struct reversemap *map, unsigned short len)
+        : outmap(map), maplength(len) {};
+
+protected:
+    /** Conversion table. */
+    const struct reversemap *outmap;
+
+    /** Length of conversion table. */
+    unsigned short maplength;
+};
+
 // Get encoder object for a character set
 Encoder *Encoder::GetEncoderByName(const string &charset)
 {
@@ -472,8 +523,8 @@ Encoder *Encoder::GetEncoderByName(const string &charset)
         if (0 == fcompare(charset, fidocharsets[i].charset))
 #endif
         {
-            return new Encoder(fidocharsets[i].outmap,
-                               fidocharsets[i].outmap_length);
+            return new SBCSEncoder(fidocharsets[i].outmap,
+                                  fidocharsets[i].outmap_length);
         }
     }
 
@@ -488,18 +539,18 @@ Encoder *Encoder::GetEncoderByName(const string &charset)
         if (0 == fcompare(charset, usenetcharsets[i].charset))
 #endif
         {
-            return new Encoder(usenetcharsets[i].outmap,
-                               usenetcharsets[i].outmap_length);
+            return new SBCSEncoder(usenetcharsets[i].outmap,
+                                   usenetcharsets[i].outmap_length);
         }
     }
 
     // Return first in MIME table as default
-    return new Encoder(usenetcharsets[0].outmap,
-                       usenetcharsets[0].outmap_length);
+    return new SBCSEncoder(usenetcharsets[0].outmap,
+                           usenetcharsets[0].outmap_length);
 }
 
 // Encode Unicode string
-string Encoder::Encode(const wstring &input)
+string SBCSEncoder::Encode(const wstring &input)
 {
     string s;
 
