@@ -1,6 +1,6 @@
-// Copyright (c) 1998-2001 Peter Karlsson
+// Copyright (c) 1998-2005 Peter Karlsson
 //
-// $Id: utility.cpp,v 1.38 2001/11/16 22:35:40 peter Exp $
+// $Id: utility.cpp,v 1.39.2.1 2005/09/05 15:19:06 peterk Exp $
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -44,9 +44,15 @@
 // Why isn't this functionality available in ANSI C++? *sigh*
 int fcompare(const string &s1, const string &s2, unsigned int max)
 {
-    int ls1 = s1.length() <? max, ls2 = s2.length() <? max;
+	size_t ls1 = s1.length();
+	if (max < ls1)
+		ls1 = max;
 
-    for (int i = 0; i < ls1 && i < ls2; i ++)
+	size_t ls2 = s2.length();
+	if (max < ls2)
+		ls2 = max;
+
+    for (size_t i = 0; i < ls1 && i < ls2; i ++)
     {
         if (toupper(s1[i]) != toupper(s2[i]))
         {
@@ -56,7 +62,7 @@ int fcompare(const string &s1, const string &s2, unsigned int max)
 
     // If we fall out, the shortest one is smallest. If we have counted to
     // max, ls1 == ls2, which gives 0.
-    return ls1 - ls2;
+    return int(ls1) - int(ls2);
 }
 
 #if !defined(HAVE_WCTYPE_H) && !defined(HAVE_TOWUPPER_IN_CTYPE_H)
@@ -79,9 +85,15 @@ inline wchar_t towupper(wchar_t wc)
 // Why isn't this functionality available in ANSI C++? *sigh*
 int fcompare(const wstring &s1, const wstring &s2, unsigned int max)
 {
-    int ls1 = s1.length() <? max, ls2 = s2.length() <? max;
+	size_t ls1 = s1.length();
+	if (max < ls1)
+		ls1 = max;
 
-    for (int i = 0; i < ls1 && i < ls2; i ++)
+	size_t ls2 = s2.length();
+	if (max < ls2)
+		ls2 = max;
+
+    for (size_t i = 0; i < ls1 && i < ls2; i ++)
     {
         if (towupper(s1[i]) != towupper(s2[i]))
         {
@@ -91,7 +103,7 @@ int fcompare(const wstring &s1, const wstring &s2, unsigned int max)
 
     // If we fall out, the shortest one is smallest. If we have counted to
     // max, ls1 == ls2, which gives 0.
-    return ls1 - ls2;
+    return int(ls1) - int(ls2);
 }
 
 // The following is from MsgEd 4.30:
@@ -127,13 +139,13 @@ time_t asciiToTimeT(const char *datetime)
 
     if (' ' == datetime[2])
     { // "Dd Mmm Yy  HH:MM:SS"
-        sscanf(datetime, "%d %s %d %d:%d:%d",
+        sscanf(datetime, "%d %3s %d %d:%d:%d",
                &tms.tm_mday, month, &tms.tm_year,
                &tms.tm_hour, &tms.tm_min, &tms.tm_sec);
     }
     else if (' ' == datetime[3])
     { // "Www Dd Mmm Yy HH:MM"
-        sscanf(&datetime[4], "%d %s %d %d:%d",
+        sscanf(&datetime[4], "%d %3s %d %d:%d",
                &tms.tm_mday, month, &tms.tm_year,
                &tms.tm_hour, &tms.tm_min);
         tms.tm_sec = 0;
@@ -171,7 +183,7 @@ time_t rfcToTimeT(string datetime)
 
     // "[ ]Dd Mmm [Yy]yy HH:MM:SS[ +ZZZZ]"
     // Note: timezones are ignored!
-    rc = sscanf(datetime.c_str(), "%d %s %d %d:%d:%d",
+    rc = sscanf(datetime.c_str(), "%d %3s %d %d:%d:%d",
                 &tms.tm_mday, month, &tms.tm_year,
                 &tms.tm_hour, &tms.tm_min, &tms.tm_sec);
 
